#include "os.h"
#include "kheap.h"
#include "task.h"
#include "initrd.h"
#include "syscall.h"
#include "shared_pages.h"
#include "pci.h"
#include "flpydsk.h"
//#include "fat12.h" //TEST

// RAM Detection by Second Stage Bootloader
#define ADDR_MEM_ENTRIES 0x0FFE
#define ADDR_MEM_INFO    0x1000

// Buffer for User-Space Program
#define FILEBUFFERSIZE   0x2000

extern page_directory_t* kernel_directory;
extern page_directory_t* current_directory;
extern ULONG file_data_start;
extern ULONG file_data_end;
ULONG address_user;
unsigned char address_TEST[FILEBUFFERSIZE];
unsigned char buf[FILEBUFFERSIZE];
unsigned char flag1 = 0; // status of user-space-program
Mem_Chunk_t Mem_Chunk[10]; // contiguous parts of memory detected by int 15h eax = 820h

extern ULONG placement_address;
heap_t* kheap = 0;

static void init()
{
    kheap = 0;
    kernel_directory  = 0;
    current_directory = 0;
    placement_address = 0x200000;

    k_clear_screen(); settextcolor(14,0);
    printformat("PrettyOS [Version 0.1.0108]\n");
    gdt_install();
    idt_install();
    isrs_install();
    irq_install();
    initODA();
    timer_install();
    keyboard_install();
}


int main()
{
    init();

    // get physical memory which is usable RAM
    USHORT num_of_entries = *( (USHORT*)(ADDR_MEM_ENTRIES) );

    #ifdef _DIAGNOSIS_
    settextcolor(2,0);
    printformat("\nNUM of RAM-Entries: %d", num_of_entries);
    settextcolor(15,0);
    #endif

    pODA->Memory_Size = 0;
    ULONG i,j;
    for(i=0; i<num_of_entries; ++i)
    {
        for(j=0; j<24; j+=4)
        {
            if(j== 0) Mem_Chunk[i].base_lo   = *( (ULONG*)(ADDR_MEM_INFO+i*24+j) );
            if(j== 4) Mem_Chunk[i].base_hi   = *( (ULONG*)(ADDR_MEM_INFO+i*24+j) );
            if(j== 8) Mem_Chunk[i].length_lo = *( (ULONG*)(ADDR_MEM_INFO+i*24+j) );
            if(j==12) Mem_Chunk[i].length_hi = *( (ULONG*)(ADDR_MEM_INFO+i*24+j) );
            if(j==16) Mem_Chunk[i].type      = *( (ULONG*)(ADDR_MEM_INFO+i*24+j) );
            if(j==20) Mem_Chunk[i].extended  = *( (ULONG*)(ADDR_MEM_INFO+i*24+j) );
        }
        if((Mem_Chunk[i].type)==1) pODA->Memory_Size += Mem_Chunk[i].length_lo;
    }
    if( (pODA->Memory_Size>0)&&(pODA->Memory_Size<=0xFFFFFFFF) )
    {
        printformat("\nUsable RAM: %d KB", (pODA->Memory_Size)/1024);
    }
    else
    {
        if(pODA->Memory_Size==0)
           pODA->Memory_Size = 0x2000000; // 32 MB

        printformat("\nMemory detection does not work. Estimated usable RAM: %d KB", (pODA->Memory_Size)/1024);
    }
    printformat("\n\n");

    // paging, kernel heap, multitasking
    paging_install();
    kheap = create_heap(KHEAP_START, KHEAP_START+KHEAP_INITIAL_SIZE, KHEAP_MAX, 1, 0); // SV and RW

    // RAM Disk
    ///
    #ifdef _DIAGNOSIS_
    settextcolor(2,0);
    printformat("rd_start: ");
    settextcolor(15,0);
    #endif
    ///
    ULONG ramdisk_start = k_malloc(0x200000, 0, 0);
    settextcolor(15,0);
    printformat("Ram Disk at: %x\n",ramdisk_start);


    // floppy disk with DMA Buffer
    // flpydsk_set_working_drive(0); // set drive 0 as current drive
	// flpydsk_install(6);           // floppy disk uses IRQ 6
	k_memset((void*)DMA_BUFFER, 0x0, 0x2400);

    tasking_install(); // ends with sti()

    /// TEST PCI-SCAN BEGIN
    ULONG pciDevVend = 0;
    ULONG pciClass   = 0;
    ULONG bus=0;
    ULONG device=0;
    ULONG func=0;

    for(bus=0;bus<8;++bus)
    {
        for(device=0;device<32;++device)
        {
            for(func=0;func<8;++func)
            {
                pciDevVend = pci_config_read_32(bus, device, func, 0x00); // register = 0x00 Device & Vendor
                pciClass   = pci_config_read_32(bus, device, func, 0x08); // register = 0x08 Class & Subclass & Interface & Revision
                if( ( pciDevVend & 0x0000FFFF ) != 0xFFFF ) // reg 0 --> Bit 0-15: vendor ID (0xFFFF: no device), Bit 16-31: device ID
                {
                    printformat("%d:%d.%d\t devID: %X venID: %X  ", bus, device, func,(pciDevVend&0xFFFF0000)>>16, (pciDevVend&0x0000FFFF));

                    if( ((pciClass>>24)==0x0C) && (((pciClass>>16)&0xFF)==0x03) ) // class: 0x0C, subclass: 0x03
                    {
                        printformat("USB ");
                        // interface
                        if( ((pciClass>>8)&0xFF)==0x00) { printformat("UHCI");   }
                        if( ((pciClass>>8)&0xFF)==0x10) { printformat("OHCI");   }
                        if( ((pciClass>>8)&0xFF)==0x20) { printformat("EHCI");   }
                        if( ((pciClass>>8)&0xFF)==0x80) { printformat("no HCI"); }
                        if( ((pciClass>>8)&0xFF)==0xFE) { printformat("any");    }
                    }
                    printformat("\n");
                }

                if( !(pci_config_read_32(bus, device, 0, 0x0C) & 0x800000) ) // Bit 7 in header type (Bit 16-23) --> multifunctional
                {
                    func = 8; // --> not multifunctional
                }
            }
        }
    }
    printformat("\n");
    /// TEST PCI-SCAN END

    // test with data and program from data.asm
    k_memcpy((void*)ramdisk_start, &file_data_start, (ULONG)&file_data_end - (ULONG)&file_data_start);
    fs_root = install_initrd(ramdisk_start);

    // search the content of files <- data from outside "loaded" via incbin ...
    settextcolor(15,0);
    i=0;
    struct dirent* node = 0;
    while( (node = readdir_fs(fs_root, i)) != 0)
    {
        fs_node_t* fsnode = finddir_fs(fs_root, node->name);

        if((fsnode->flags & 0x7) == FS_DIRECTORY)
        {
            printformat("<RAM Disk DIR> %s\n",node->name);
        }
        else
        {
            ULONG sz = read_fs(fsnode, 0, fsnode->length, buf);


            char name[40];
            k_memset((void*)name, 0, 40);
            k_memcpy((void*)name, node->name, 35); // protection against wrong / too long filename
            printformat("%d \t%s\n",sz,name);

            ULONG j;
            if( k_strcmp(node->name,"shell")==0 )
            {
                flag1=1;

                if(sz>=FILEBUFFERSIZE)
                {
                    sz = 0;
                    settextcolor(4,0);
                    printformat("Buffer Overflow prohibited!\n");
                    settextcolor(15,0);
                }

                for(j=0; j<sz; ++j)
                    address_TEST[j] = buf[j];
            }
        }
        ++i;
    }
    printformat("\n");

        // shell in elf-executable-format provided by data.asm
    ULONG elf_vaddr     = *( (ULONG*)( address_TEST + 0x3C ) );
    ULONG elf_offset    = *( (ULONG*)( address_TEST + 0x38 ) );
    ULONG elf_filesize  = *( (ULONG*)( address_TEST + 0x44 ) );

    ///
    #ifdef _DIAGNOSIS_
    settextcolor(5,0);
    printformat("virtual address: %x offset: %x size: %x", elf_vaddr, elf_offset, elf_filesize);
    printformat("\n\n");
    settextcolor(15,0);
    #endif
    ///

    if( (elf_vaddr>0x5FF000) || (elf_vaddr<0x400000) || (elf_offset>0x130) || (elf_filesize>0x1000) )
    {
        flag1=2;
    }

    // Test-Programm ==> user space
    if(flag1==1)
    {
        address_user = elf_vaddr;
        k_memcpy((void*)address_user, address_TEST + elf_offset, elf_filesize); // ELF LOAD: Offset VADDR FileSize
    }

    pODA->ts_flag = 1; // enable task_switching

    if(flag1==1)
    {
      create_task ((void*)getAddressUserProgram(),3); // program in user space (ring 3) takes over
    }
    else
    {
        if(flag1==0)
        {
            settextcolor(4,0);
            printformat("Program not found.\n");
            settextcolor(15,0);
        }
        else
        {
            settextcolor(4,0);
            printformat("Program corrupt.\n");
            settextcolor(15,0);
        }
    }

    while(TRUE){/*  */}
    return 0;
}
